"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Protocol = exports.CPI_SEED = exports.ACCOUNT_SEED = exports.STATE_SEED = void 0;
const web3_js_1 = require("@solana/web3.js");
const anchor_1 = require("@project-serum/anchor");
const fabric_synths_json_1 = __importDefault(require("./idl/fabric_synths.json"));
const fabric_faucet_json_1 = __importDefault(require("./idl/fabric_faucet.json"));
const spl_token_1 = require("@solana/spl-token");
const spl_token_2 = require("@solana/spl-token");
// Constants
exports.STATE_SEED = "fstate1";
exports.ACCOUNT_SEED = "faccv1";
exports.CPI_SEED = "FABRIC";
// 
class Protocol {
    constructor(connection, wallet, programId, faucetProgramId, opts) {
        this.idl = fabric_synths_json_1.default;
        this.faucetIdl = fabric_faucet_json_1.default;
        this.wallet = wallet;
        this.programId = programId;
        this.connection = connection;
        const provider = new anchor_1.Provider(this.connection, this.wallet, opts || anchor_1.Provider.defaultOptions());
        this.program = new anchor_1.Program(fabric_synths_json_1.default, this.programId, provider);
        this.faucetProgram = new anchor_1.Program(fabric_faucet_json_1.default, faucetProgramId, provider);
    }
    static initialise(connection, wallet, programId, faucetProgramId, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            var protocol = new Protocol(connection, wallet, programId, faucetProgramId, opts);
            protocol.statePubkey = (yield protocol.getStateAccount())[0];
            try {
                yield protocol.getState();
            }
            catch (_a) { }
            return protocol;
        });
    }
    // ix methods
    initialiseContract(epochDurationInSlots, rewardTokenAAccount, rewardWithdrawlDelay, rewardTokenAPerEpoch) {
        return __awaiter(this, void 0, void 0, function* () {
            var stateAddressInfo = yield this.getStateAccount();
            this.adminPubkey = this.wallet.publicKey;
            this.statePubkey = stateAddressInfo[0];
            let cpiAccountInfo = yield this.getCpiAuthAccount();
            this.cpiAuthPubkey = cpiAccountInfo[0];
            let epochDuration = new anchor_1.BN(epochDurationInSlots);
            var tx = yield this.program.rpc.initialiseContract(new anchor_1.BN(stateAddressInfo[1]), new anchor_1.BN(cpiAccountInfo[1]), epochDuration, new anchor_1.BN(rewardWithdrawlDelay || 0), new anchor_1.BN(rewardTokenAPerEpoch * 1e8 || 0), new anchor_1.BN(8), {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    rewardTokenAAccount: rewardTokenAAccount
                }
            });
            return tx;
        });
    }
    initialiseAssetsList(collateralMintPubkey, collateralFeedPubkey, collateralReserveAccountPubkey, collateralLiquidationFundAccountPubkey) {
        return __awaiter(this, void 0, void 0, function* () {
            const assetListAccount = web3_js_1.Keypair.generate();
            var stateAddressInfo = yield this.getStateAccount();
            let usdToken = yield this.createToken(8, this.cpiAuthPubkey);
            let tx = yield this.program.rpc.initialiseAssetsList({
                accounts: {
                    stateAccount: stateAddressInfo[0],
                    assetsList: assetListAccount.publicKey,
                    adminAccount: this.wallet.publicKey,
                    collateralToken: collateralMintPubkey,
                    collateralTokenFeed: collateralFeedPubkey,
                    usdToken: usdToken.publicKey,
                    fabReserve: collateralReserveAccountPubkey,
                    fabLiquidationFund: collateralLiquidationFundAccountPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY
                },
                signers: [
                    assetListAccount,
                    // @ts-expect-error
                    this.program.provider.wallet.payer,
                ],
                instructions: [
                    yield this.program.account.assetsList.createInstruction(assetListAccount)
                ]
            });
            this.assetsListPubkey = assetListAccount.publicKey;
            let result = [
                tx,
                usdToken,
                assetListAccount.publicKey
            ];
            return result;
        });
    }
    setAssetsListAccount(assetListPubkey) {
        return __awaiter(this, void 0, void 0, function* () {
            let tx = yield this.program.rpc.setAssetsList({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsList: assetListPubkey
                }
            });
            this.assetsListPubkey = assetListPubkey;
            return tx;
        });
    }
    depositCollateral(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var accounts = {
                accounts: {
                    stateAccount: this.statePubkey,
                    owner,
                    userAccount: userAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    adminAccount: this.adminPubkey,
                    collateralReserveAccount,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ]
            };
            let tx = yield this.program.rpc.deposit(amount, accounts);
            return tx;
        });
    }
    depositCollateralIx(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var accounts = {
                accounts: {
                    stateAccount: this.statePubkey,
                    owner,
                    userAccount: userAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    adminAccount: this.adminPubkey,
                    collateralReserveAccount,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            };
            return (yield this.program.instruction.depositCollateral(amount, accounts));
        });
    }
    mint(amount, owner, fusdReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            let tx = yield this.program.rpc.mint(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    toAccount: fusdReceiver,
                    userAccount: userAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    owner,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ]
            });
            return tx;
        });
    }
    burn(amount, owner, fusdReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.burn(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    assetsListAccount: this.assetsListPubkey,
                    adminAccount: this.adminPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    fusdToBurnAccount: fusdReceiver,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    userAccount: userAccount,
                    owner: owner,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ]
            });
            return tx;
        });
    }
    withdrawCollateral(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.withdraw(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    userAccount: userAccount,
                    collateralReserveAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ]
            });
            return tx;
        });
    }
    swap(amount, owner, tokenAMint, tokenBMint, tokenAReceiver, tokenBReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.swap(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    tokenInAccount: tokenAMint,
                    tokenOutAccount: tokenBMint,
                    userTokenInAccount: tokenAReceiver,
                    userTokenOutAccount: tokenBReceiver,
                    userAccount,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            });
            return tx;
        });
    }
    claimRewards(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.claimRewards({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    userAccount
                }
            });
            return tx;
        });
    }
    withdrawRewards(userAccount, owner, rewardTokenAReceiver) {
        return __awaiter(this, void 0, void 0, function* () {
            var rewardTokenARewardsAccount = (yield this.getState()).globalStakingData.rewardTokenAAccount;
            var tx = yield this.program.rpc.withdrawRewards({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    userAccount,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    owner,
                    rewardTokenAAccount: rewardTokenARewardsAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    usersRewardTokenAAccount: rewardTokenAReceiver
                }
            });
            return tx;
        });
    }
    updatePrices() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var feedAddresses = assetList.assets
                .filter((asset) => !asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0)))
                .map((asset) => {
                return {
                    pubkey: asset.priceFeedAddress,
                    isWritable: false,
                    isSigner: false
                };
            });
            var tx = yield this.program.rpc.setAssetPrices({
                accounts: {
                    assetsList: this.assetsListPubkey,
                },
                remainingAccounts: feedAddresses
            });
            return tx;
        });
    }
    updatePricesIx() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var feedAddresses = assetList.assets
                .filter((asset) => !asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0)))
                .map((asset) => {
                return {
                    pubkey: asset.priceFeedAddress,
                    isWritable: false,
                    isSigner: false
                };
            });
            var ix = yield this.program.instruction.setAssetPrices({
                accounts: {
                    assetsList: this.assetsListPubkey,
                },
                remainingAccounts: feedAddresses
            });
            return ix;
        });
    }
    // helper methods
    getStateAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            const [stateAddress, stateBump] = yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode(exports.STATE_SEED))], this.program.programId);
            var state = [
                stateAddress,
                stateBump
            ];
            return state;
        });
    }
    ;
    getState() {
        return __awaiter(this, void 0, void 0, function* () {
            const state = yield this.program.account.state.fetch(this.statePubkey);
            this.adminPubkey = state.adminPubkey;
            this.assetsListPubkey = state.assetsList;
            this.cpiAuthPubkey = state.cpiAuthPubkey;
            return state;
        });
    }
    getAssetList() {
        return __awaiter(this, void 0, void 0, function* () {
            const state = yield this.getState();
            const assetsList = yield this.program.account.assetsList.fetch(state.assetsList);
            assetsList.assets = assetsList.assets.slice(0, assetsList.numberOfAssets);
            assetsList.collaterals = assetsList.collaterals.slice(0, assetsList.numberOfCollaterals);
            assetsList.synthetics = assetsList.synthetics.slice(0, assetsList.numberOfSynthetics);
            return assetsList;
        });
    }
    createUserAccount(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var [userAddress, userBump] = yield this.getUserAccount(owner);
            var tx = yield this.program.rpc.createUserAccount(userBump, {
                accounts: {
                    userAccount: userAddress,
                    adminAccount: owner,
                    payer: this.wallet.publicKey,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    systemProgram: web3_js_1.SystemProgram.programId,
                }
            });
            yield this.program.provider.connection.confirmTransaction(tx);
            return userAddress;
        });
    }
    getUserAccount(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            const [account, bump] = yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode(exports.ACCOUNT_SEED)), owner.toBuffer()], this.program.programId);
            let result = [
                account,
                bump
            ];
            return result;
        });
    }
    getUser(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.program.account.userAccount.fetch(userAccount);
            return user;
        });
    }
    checkIfUserAccountExists(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            const userAddress = yield this.getUserAccount(owner);
            try {
                yield this.getUser(userAddress[0]);
                return true;
            }
            catch (_a) {
                return false;
            }
        });
    }
    getCpiAuthAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            const [cpiAccount, cpiBump] = yield web3_js_1.PublicKey.findProgramAddress([
                Buffer.from(exports.CPI_SEED),
            ], this.programId);
            let result = [
                cpiAccount,
                cpiBump
            ];
            return result;
        });
    }
    mintTokens(amount, decimals, toAddress, token, authority, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (authority === undefined) {
                authority = this.wallet.publicKey;
            }
            var signers = [];
            if (signer !== undefined) {
                signers.push(signer);
            }
            // you must be the correct authority to mint tokens
            var amountToMint = new anchor_1.BN(amount * Math.pow(10, decimals));
            yield token.mintTo(toAddress, authority, signers, new spl_token_1.u64(amountToMint.toString()));
        });
    }
    createTokenAccount(token, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var address = yield token.createAccount(owner);
            return address;
        });
    }
    getAccountBalanceForMint(tokenAccount) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            var accountInfo = yield this.connection.getParsedAccountInfo(tokenAccount);
            // @ts-expect-error
            var balance = (_a = accountInfo.value) === null || _a === void 0 ? void 0 : _a.data.parsed.info.tokenAmount.uiAmount;
            balance = (balance !== undefined) ? balance : 0;
            return balance;
        });
    }
    getTokenAccountInfo(token, tokenAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const info = yield token.getAccountInfo(tokenAccount);
            return info;
        });
    }
    getfUsdTokenMint() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            return assetList.synthetics[0].assetAddress;
        });
    }
    createToken(decimals, authority) {
        return __awaiter(this, void 0, void 0, function* () {
            if (authority === undefined) {
                authority = this.wallet.publicKey;
            }
            const token = yield spl_token_1.Token.createMint(this.connection, 
            // @ts-expect-error
            this.program.provider.wallet.payer, authority, null, decimals, spl_token_1.TOKEN_PROGRAM_ID);
            return token;
        });
    }
    getUserCollateralRatio(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const userDebtUsd = yield this.getUserDebtUsd(userAccount);
            const userCollateralUsd = yield this.getUserCollateralUsd(userAccount);
            if (userDebtUsd === 0 && userCollateralUsd === 0) {
                return 0;
            }
            if (userDebtUsd === 0 && userCollateralUsd !== 0) {
                return Infinity;
            }
            const userCollateralRatio = (userCollateralUsd / userDebtUsd);
            return userCollateralRatio;
        });
    }
    getUserDebtUsd(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.getUser(userAccount);
            const state = yield this.getState();
            const globalDebtUsd = yield this.getGlobalDebtUsd();
            const userDebtShares = (user.userDebtShares).toNumber();
            const globalDebtShares = state.debtShares;
            if (globalDebtShares.toNumber() === 0 || userDebtShares === 0) {
                return 0;
            }
            return (globalDebtUsd * userDebtShares / globalDebtShares.toNumber());
        });
    }
    getUserCollateralUsd(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.getUser(userAccount);
            const assetList = yield this.getAssetList();
            const assets = assetList.assets;
            const collaterals = assetList.collaterals;
            const userCollaterals = user.userCollaterals;
            var sum = new anchor_1.BN(0);
            userCollaterals.forEach(userCollateral => {
                const collateral = collaterals[userCollateral.index];
                const asset = assets[collateral.assetIndex];
                // TODO: refactor to div from price feed exponent + scale of price
                var increment = (new anchor_1.BN(0)).add(userCollateral.amount
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.reserveBalance.scale)))
                    .mul(asset.price.val))
                    .div(new anchor_1.BN(10).pow((new anchor_1.BN(asset.price.scale)).abs()));
                sum = sum.add(increment);
            });
            return (sum.toNumber());
        });
    }
    getMaximumPossibleUserDebtUsd(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.getUser(userAccount);
            const assetList = yield this.getAssetList();
            const assets = assetList.assets;
            const collaterals = assetList.collaterals;
            const userCollaterals = user.userCollaterals;
            var sum = new anchor_1.BN(0);
            userCollaterals.forEach(userCollateral => {
                const collateral = collaterals[userCollateral.index];
                const asset = assets[collateral.assetIndex];
                // TODO: refactor to div from price feed exponent + scale of price
                sum = sum.add(userCollateral.amount
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.reserveBalance.scale)))
                    .mul(asset.price.val)
                    .div(new anchor_1.BN(10).pow((new anchor_1.BN(asset.price.scale)).abs()))
                    .mul(collateral.collateralRatio.val)
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.collateralRatio.scale))));
            });
            return (sum.toNumber());
        });
    }
    getGlobalDebtUsd() {
        return __awaiter(this, void 0, void 0, function* () {
            const assetList = yield this.getAssetList();
            var sum = new anchor_1.BN(0);
            assetList.synthetics.forEach(element => {
                const asset = assetList.assets[element.assetIndex];
                sum = sum.add(element.supply.val
                    .mul(asset.price.val)
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(element.supply.scale)))
                    .div(new anchor_1.BN(Math.pow(10, 8))));
            });
            return sum.toNumber();
        });
    }
    getSyntheticSupplyAndMarketCap(mint) {
        return __awaiter(this, void 0, void 0, function* () {
            const assetList = yield this.getAssetList();
            const synthetic = assetList.synthetics.find(synth => synth.assetAddress.toBase58() === mint.toBase58());
            const asset = assetList.assets[synthetic.assetIndex];
            var supply = synthetic.supply.val.toNumber();
            supply = (supply === 0) ? 0 : supply / (Math.pow(10, synthetic.supply.scale));
            const price = asset.price.val.toNumber();
            const scaledPrice = price / (Math.pow(10, Math.abs(asset.price.scale)));
            const marketCap = supply * scaledPrice;
            return {
                supply,
                scaledPrice,
                marketCap
            };
        });
    }
    getCollateralTvlAndCRatio(mint) {
        return __awaiter(this, void 0, void 0, function* () {
            const assetList = yield this.getAssetList();
            const collateral = assetList.collaterals.find(collateral => collateral.collateralMintAddress.toBase58() === mint.toBase58());
            const asset = assetList.assets[collateral.assetIndex];
            var collateralRatio = collateral.collateralRatio.val.toNumber();
            collateralRatio = (collateralRatio === 0) ? 0 : collateralRatio / (Math.pow(10, collateral.collateralRatio.scale));
            var tvl = collateral.reserveBalance.val.toNumber();
            tvl = (tvl === 0) ? 0 : tvl / (Math.pow(10, collateral.reserveBalance.scale));
            const price = asset.price.val.toNumber();
            const scaledPrice = price / (Math.pow(10, Math.abs(asset.price.scale)));
            const collateralTvl = tvl * scaledPrice;
            return {
                collateralTvl,
                collateralRatio
            };
        });
    }
    // TODO: functions: 
    // calculate global debt
    // calculate user debt
    // add synthetic
    // TODO: internal methods
    // create price feed
    // TODO: liquidate ix
    haltProtocol() {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateHalted(true, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    unhaltProtocol() {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateHalted(false, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    updateEpochDuration(durationInSlots) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateEpochDuration(new anchor_1.BN(durationInSlots), {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    updateRewardTokenAAccount(rewardTokenAAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateRewardTokenAAccount({
                accounts: {
                    newRewardAccount: rewardTokenAAccount,
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                }
            });
            return tx;
        });
    }
    airdropFab(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            // find mint pda and bump
            const [pda, bump] = yield this.getFabAirdropMintProgramAddress();
            // find associated token account
            const associatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_2.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, pda, owner, true);
            const airdropAmount = new anchor_1.BN(100000 * 1e8);
            var tx = yield this.faucetProgram.rpc.airdrop(bump, airdropAmount, {
                accounts: {
                    mint: pda,
                    receiver: owner,
                    destination: associatedTokenAccount,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    associatedTokenProgram: spl_token_2.ASSOCIATED_TOKEN_PROGRAM_ID,
                    payer: this.wallet.publicKey
                }
            });
            return tx;
        });
    }
    airdropFabDirectly(owner, fabAccount, amount = 100000) {
        return __awaiter(this, void 0, void 0, function* () {
            // find mint pda and bump
            const [pda, bump] = yield this.getFabAirdropMintProgramAddress();
            const airdropAmount = new anchor_1.BN(amount * 1e8);
            var tx = yield this.faucetProgram.rpc.airdrop(bump, airdropAmount, {
                accounts: {
                    mint: pda,
                    receiver: owner,
                    destination: fabAccount,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    associatedTokenProgram: spl_token_2.ASSOCIATED_TOKEN_PROGRAM_ID,
                    payer: this.wallet.publicKey
                }
            });
            return tx;
        });
    }
    getFabAirdropMintProgramAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode("FABRIC-FAUCET-FAB"))], this.faucetProgram.programId);
        });
    }
}
exports.Protocol = Protocol;
;
;
